@@ -11,18 +11,14 @@
 """
 
 import warnings
+from typing import Any, Dict
 
 from sphinxcontrib.devhelp import DevhelpBuilder
 
+from sphinx.application import Sphinx
 from sphinx.deprecation import RemovedInSphinx40Warning, deprecated_alias
 
 
-if False:
-    # For type annotation
-    from typing import Any, Dict  # NOQA
-    from sphinx.application import Sphinx  # NOQA
-
-
 deprecated_alias('sphinx.builders.devhelp',
                  {
                      'DevhelpBuilder': DevhelpBuilder,
@@ -30,8 +26,7 @@ deprecated_alias('sphinx.builders.devhelp',
                  RemovedInSphinx40Warning)
 
 
-def setup(app):
-    # type: (Sphinx) -> Dict[str, Any]
+def setup(app: Sphinx) -> Dict[str, Any]:
     warnings.warn('sphinx.builders.devhelp has been moved to sphinxcontrib-devhelp.',
                   RemovedInSphinx40Warning)
     app.setup_extension('sphinxcontrib.devhelp')
