@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.ext.coverage
     ~~~~~~~~~~~~~~~~~~~
@@ -6,122 +5,131 @@
     Check Python modules and C API for coverage.  Mostly written by Josip
     Dzolonga for the Google Highly Open Participation contest.
 
-    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-import re
 import glob
 import inspect
+import pickle
+import re
+from importlib import import_module
 from os import path
-
-from six import iteritems
-from six.moves import cPickle as pickle
+from typing import Any, Dict, IO, List, Pattern, Set, Tuple
 
 import sphinx
+from sphinx.application import Sphinx
 from sphinx.builders import Builder
+from sphinx.locale import __
+from sphinx.util import logging
+from sphinx.util.inspect import safe_getattr
+
+logger = logging.getLogger(__name__)
 
 
 # utility
-def write_header(f, text, char='-'):
+def write_header(f: IO, text: str, char: str = '-') -> None:
     f.write(text + '\n')
     f.write(char * len(text) + '\n')
 
 
-def compile_regex_list(name, exps, warnfunc):
+def compile_regex_list(name: str, exps: str) -> List[Pattern]:
     lst = []
     for exp in exps:
         try:
             lst.append(re.compile(exp))
         except Exception:
-            warnfunc('invalid regex %r in %s' % (exp, name))
+            logger.warning(__('invalid regex %r in %s'), exp, name)
     return lst
 
 
 class CoverageBuilder(Builder):
-
+    """
+    Evaluates coverage of code in the documentation.
+    """
     name = 'coverage'
+    epilog = __('Testing of coverage in the sources finished, look at the '
+                'results in %(outdir)s' + path.sep + 'python.txt.')
 
-    def init(self):
-        self.c_sourcefiles = []
+    def init(self) -> None:
+        self.c_sourcefiles = []  # type: List[str]
         for pattern in self.config.coverage_c_path:
             pattern = path.join(self.srcdir, pattern)
             self.c_sourcefiles.extend(glob.glob(pattern))
 
-        self.c_regexes = []
+        self.c_regexes = []  # type: List[Tuple[str, Pattern]]
         for (name, exp) in self.config.coverage_c_regexes.items():
             try:
                 self.c_regexes.append((name, re.compile(exp)))
             except Exception:
-                self.warn('invalid regex %r in coverage_c_regexes' % exp)
+                logger.warning(__('invalid regex %r in coverage_c_regexes'), exp)
 
-        self.c_ignorexps = {}
-        for (name, exps) in iteritems(self.config.coverage_ignore_c_items):
-            self.c_ignorexps[name] = compile_regex_list(
-                'coverage_ignore_c_items', exps, self.warn)
-        self.mod_ignorexps = compile_regex_list(
-            'coverage_ignore_modules', self.config.coverage_ignore_modules,
-            self.warn)
-        self.cls_ignorexps = compile_regex_list(
-            'coverage_ignore_classes', self.config.coverage_ignore_classes,
-            self.warn)
-        self.fun_ignorexps = compile_regex_list(
-            'coverage_ignore_functions', self.config.coverage_ignore_functions,
-            self.warn)
+        self.c_ignorexps = {}  # type: Dict[str, List[Pattern]]
+        for (name, exps) in self.config.coverage_ignore_c_items.items():
+            self.c_ignorexps[name] = compile_regex_list('coverage_ignore_c_items',
+                                                        exps)
+        self.mod_ignorexps = compile_regex_list('coverage_ignore_modules',
+                                                self.config.coverage_ignore_modules)
+        self.cls_ignorexps = compile_regex_list('coverage_ignore_classes',
+                                                self.config.coverage_ignore_classes)
+        self.fun_ignorexps = compile_regex_list('coverage_ignore_functions',
+                                                self.config.coverage_ignore_functions)
+        self.py_ignorexps = compile_regex_list('coverage_ignore_pyobjects',
+                                               self.config.coverage_ignore_pyobjects)
 
-    def get_outdated_docs(self):
+    def get_outdated_docs(self) -> str:
         return 'coverage overview'
 
-    def write(self, *ignored):
-        self.py_undoc = {}
+    def write(self, *ignored) -> None:
+        self.py_undoc = {}  # type: Dict[str, Dict[str, Any]]
         self.build_py_coverage()
         self.write_py_coverage()
 
-        self.c_undoc = {}
+        self.c_undoc = {}  # type: Dict[str, Set[Tuple[str, str]]]
         self.build_c_coverage()
         self.write_c_coverage()
 
-    def build_c_coverage(self):
+    def build_c_coverage(self) -> None:
         # Fetch all the info from the header files
         c_objects = self.env.domaindata['c']['objects']
         for filename in self.c_sourcefiles:
-            undoc = set()
-            f = open(filename, 'r')
-            try:
+            undoc = set()  # type: Set[Tuple[str, str]]
+            with open(filename) as f:
                 for line in f:
                     for key, regex in self.c_regexes:
                         match = regex.match(line)
                         if match:
                             name = match.groups()[0]
                             if name not in c_objects:
-                                for exp in self.c_ignorexps.get(key, ()):
+                                for exp in self.c_ignorexps.get(key, []):
                                     if exp.match(name):
                                         break
                                 else:
                                     undoc.add((key, name))
                             continue
-            finally:
-                f.close()
             if undoc:
                 self.c_undoc[filename] = undoc
 
-    def write_c_coverage(self):
+    def write_c_coverage(self) -> None:
         output_file = path.join(self.outdir, 'c.txt')
-        op = open(output_file, 'w')
-        try:
+        with open(output_file, 'w') as op:
             if self.config.coverage_write_headline:
                 write_header(op, 'Undocumented C API elements', '=')
             op.write('\n')
 
-            for filename, undoc in iteritems(self.c_undoc):
+            for filename, undoc in self.c_undoc.items():
                 write_header(op, filename)
                 for typ, name in sorted(undoc):
                     op.write(' * %-50s [%9s]\n' % (name, typ))
                 op.write('\n')
-        finally:
-            op.close()
 
-    def build_py_coverage(self):
+    def ignore_pyobj(self, full_name):
+        for exp in self.py_ignorexps:
+            if exp.search(full_name):
+                return True
+        return False
+
+    def build_py_coverage(self) -> None:
         objects = self.env.domaindata['py']['objects']
         modules = self.env.domaindata['py']['modules']
 
@@ -133,19 +141,18 @@ class CoverageBuilder(Builder):
                 if exp.match(mod_name):
                     ignore = True
                     break
-            if ignore:
+            if ignore or self.ignore_pyobj(mod_name):
                 continue
 
             try:
-                mod = __import__(mod_name, fromlist=['foo'])
+                mod = import_module(mod_name)
             except ImportError as err:
-                self.warn('module %s could not be imported: %s' %
-                          (mod_name, err))
+                logger.warning(__('module %s could not be imported: %s'), mod_name, err)
                 self.py_undoc[mod_name] = {'error': err}
                 continue
 
             funcs = []
-            classes = {}
+            classes = {}  # type: Dict[str, List[str]]
 
             for name, obj in inspect.getmembers(mod):
                 # diverse module attributes are ignored:
@@ -160,6 +167,8 @@ class CoverageBuilder(Builder):
                     continue
 
                 full_name = '%s.%s' % (mod_name, name)
+                if self.ignore_pyobj(full_name):
+                    continue
 
                 if inspect.isfunction(obj):
                     if full_name not in objects:
@@ -182,12 +191,15 @@ class CoverageBuilder(Builder):
                             classes[name] = []
                             continue
 
-                        attrs = []
+                        attrs = []  # type: List[str]
 
                         for attr_name in dir(obj):
                             if attr_name not in obj.__dict__:
                                 continue
-                            attr = getattr(obj, attr_name)
+                            try:
+                                attr = safe_getattr(obj, attr_name)
+                            except AttributeError:
+                                continue
                             if not (inspect.ismethod(attr) or
                                     inspect.isfunction(attr)):
                                 continue
@@ -197,22 +209,21 @@ class CoverageBuilder(Builder):
                             if skip_undoc and not attr.__doc__:
                                 # skip methods without docstring if wished
                                 continue
-
                             full_attr_name = '%s.%s' % (full_name, attr_name)
+                            if self.ignore_pyobj(full_attr_name):
+                                continue
                             if full_attr_name not in objects:
                                 attrs.append(attr_name)
-
                         if attrs:
                             # some attributes are undocumented
                             classes[name] = attrs
 
             self.py_undoc[mod_name] = {'funcs': funcs, 'classes': classes}
 
-    def write_py_coverage(self):
+    def write_py_coverage(self) -> None:
         output_file = path.join(self.outdir, 'python.txt')
-        op = open(output_file, 'w')
         failed = []
-        try:
+        with open(output_file, 'w') as op:
             if self.config.coverage_write_headline:
                 write_header(op, 'Undocumented Python objects', '=')
             keys = sorted(self.py_undoc.keys())
@@ -232,7 +243,7 @@ class CoverageBuilder(Builder):
                     if undoc['classes']:
                         op.write('Classes:\n')
                         for name, methods in sorted(
-                                iteritems(undoc['classes'])):
+                                undoc['classes'].items()):
                             if not methods:
                                 op.write(' * %s\n' % name)
                             else:
@@ -243,27 +254,23 @@ class CoverageBuilder(Builder):
             if failed:
                 write_header(op, 'Modules that failed to import')
                 op.writelines(' * %s -- %s\n' % x for x in failed)
-        finally:
-            op.close()
 
-    def finish(self):
+    def finish(self) -> None:
         # dump the coverage data to a pickle file too
         picklepath = path.join(self.outdir, 'undoc.pickle')
-        dumpfile = open(picklepath, 'wb')
-        try:
+        with open(picklepath, 'wb') as dumpfile:
             pickle.dump((self.py_undoc, self.c_undoc), dumpfile)
-        finally:
-            dumpfile.close()
 
 
-def setup(app):
+def setup(app: Sphinx) -> Dict[str, Any]:
     app.add_builder(CoverageBuilder)
     app.add_config_value('coverage_ignore_modules', [], False)
     app.add_config_value('coverage_ignore_functions', [], False)
     app.add_config_value('coverage_ignore_classes', [], False)
+    app.add_config_value('coverage_ignore_pyobjects', [], False)
     app.add_config_value('coverage_c_path', [], False)
     app.add_config_value('coverage_c_regexes', {}, False)
     app.add_config_value('coverage_ignore_c_items', {}, False)
     app.add_config_value('coverage_write_headline', True, False)
     app.add_config_value('coverage_skip_undoc_in_source', False, False)
-    return {'version': sphinx.__version__, 'parallel_read_safe': True}
+    return {'version': sphinx.__display_version__, 'parallel_read_safe': True}
