@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.ext.ifconfig
     ~~~~~~~~~~~~~~~~~~~
@@ -16,41 +15,44 @@
     namespace of the project configuration (that is, all variables from
     ``conf.py`` are available.)
 
-    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
+from typing import Any, Dict, List
+
 from docutils import nodes
+from docutils.nodes import Node
 
 import sphinx
-from sphinx.util.nodes import set_source_info
-from sphinx.util.compat import Directive
+from sphinx.application import Sphinx
+from sphinx.util.docutils import SphinxDirective
+from sphinx.util.nodes import nested_parse_with_titles
 
 
 class ifconfig(nodes.Element):
     pass
 
 
-class IfConfig(Directive):
+class IfConfig(SphinxDirective):
 
     has_content = True
     required_arguments = 1
     optional_arguments = 0
     final_argument_whitespace = True
-    option_spec = {}
+    option_spec = {}  # type: Dict
 
-    def run(self):
+    def run(self) -> List[Node]:
         node = ifconfig()
         node.document = self.state.document
-        set_source_info(self, node)
+        self.set_source_info(node)
         node['expr'] = self.arguments[0]
-        self.state.nested_parse(self.content, self.content_offset,
-                                node, match_titles=1)
+        nested_parse_with_titles(self.state, self.content, node)
         return [node]
 
 
-def process_ifconfig_nodes(app, doctree, docname):
-    ns = dict((k, app.config[k]) for k in app.config.values)
+def process_ifconfig_nodes(app: Sphinx, doctree: nodes.document, docname: str) -> None:
+    ns = {confval.name: confval.value for confval in app.config}
     ns.update(app.config.__dict__.copy())
     ns['builder'] = app.builder.name
     for node in doctree.traverse(ifconfig):
@@ -71,8 +73,8 @@ def process_ifconfig_nodes(app, doctree, docname):
                 node.replace_self(node.children)
 
 
-def setup(app):
+def setup(app: Sphinx) -> Dict[str, Any]:
     app.add_node(ifconfig)
     app.add_directive('ifconfig', IfConfig)
     app.connect('doctree-resolved', process_ifconfig_nodes)
-    return {'version': sphinx.__version__, 'parallel_read_safe': True}
+    return {'version': sphinx.__display_version__, 'parallel_read_safe': True}
