@@ -10,6 +10,7 @@
 
 import traceback
 import warnings
+from importlib import import_module
 from inspect import isclass
 from types import MethodType
 
@@ -30,7 +31,8 @@ from sphinx.util.logging import prefixed_warnings
 
 if False:
     # For type annotation
-    from typing import Any, Callable, Dict, Iterator, List, Tuple, Type, Union  # NOQA
+    from typing import Any, Callable, Dict, Iterator, List, Tuple, Union  # NOQA
+    from typing import Type  # for python3.5.1
     from docutils import nodes  # NOQA
     from docutils.io import Input  # NOQA
     from docutils.parsers import Parser  # NOQA
@@ -470,18 +472,19 @@ class SphinxComponentRegistry:
         prefix = __('while setting up extension %s:') % extname
         with prefixed_warnings(prefix):
             try:
-                mod = __import__(extname, None, None, ['setup'])
+                mod = import_module(extname)
             except ImportError as err:
                 logger.verbose(__('Original exception:\n') + traceback.format_exc())
                 raise ExtensionError(__('Could not import extension %s') % extname, err)
 
-            if not hasattr(mod, 'setup'):
+            setup = getattr(mod, 'setup', None)
+            if setup is None:
                 logger.warning(__('extension %r has no setup() function; is it really '
                                   'a Sphinx extension module?'), extname)
                 metadata = {}  # type: Dict[str, Any]
             else:
                 try:
-                    metadata = mod.setup(app)
+                    metadata = setup(app)
                 except VersionRequirementError as err:
                     # add the extension name to the version required
                     raise VersionRequirementError(
