@@ -1,29 +1,19 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.search.en
     ~~~~~~~~~~~~~~~~
 
     English search language: includes the JS porter stemmer.
 
-    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 from sphinx.search import SearchLanguage
+from sphinx.util.stemmer import get_stemmer
 
-try:
-    # http://bitbucket.org/methane/porterstemmer/
-    from porterstemmer import Stemmer as CStemmer
-    CSTEMMER = True
-    PYSTEMMER = False
-except ImportError:
-    CSTEMMER = False
-    try:
-        from Stemmer import Stemmer as PyStemmer
-        PYSTEMMER = True
-    except ImportError:
-        from sphinx.util.stemmer import PorterStemmer
-        PYSTEMMER = False
+if False:
+    # For type annotation
+    from typing import Dict  # NOQA
 
 english_stopwords = set("""
 a  and  are  as  at
@@ -231,27 +221,9 @@ class SearchEnglish(SearchLanguage):
     stopwords = english_stopwords
 
     def init(self, options):
-        if CSTEMMER:
-            class Stemmer(CStemmer):
-                def stem(self, word):
-                    return self(word.lower())
-        elif PYSTEMMER:
-            class Stemmer(object):
-                def __init__(self):
-                    self.stemmer = PyStemmer('porter')
-
-                def stem(self, word):
-                    return self.stemmer.stemWord(word)
-        else:
-            class Stemmer(PorterStemmer):
-                """All those porter stemmer implementations look hideous;
-                make at least the stem method nicer.
-                """
-                def stem(self, word):
-                    word = word.lower()
-                    return PorterStemmer.stem(self, word, 0, len(word) - 1)
-
-        self.stemmer = Stemmer()
+        # type: (Dict) -> None
+        self.stemmer = get_stemmer()
 
     def stem(self, word):
-        return self.stemmer.stem(word)
+        # type: (str) -> str
+        return self.stemmer.stem(word.lower())
