@@ -1,14 +1,14 @@
-# -*- coding: utf-8 -*-
-
-import sys
 import os
+import sys
 
-sys.path.insert(0, os.path.abspath('.'))
+source_dir = os.path.abspath('.')
+if source_dir not in sys.path:
+    sys.path.insert(0, source_dir)
 extensions = ['sphinx.ext.autodoc', 'sphinx.ext.viewcode']
-master_doc = 'index'
+exclude_patterns = ['_build']
 
 
-if 'test_linkcode' in tags:
+if 'test_linkcode' in tags:  # NOQA
     extensions.remove('sphinx.ext.viewcode')
     extensions.append('sphinx.ext.linkcode')
 
