@@ -1,15 +1,17 @@
-# -*- coding: utf-8 -*-
 """
     test_toctree
     ~~~~~~~~~~~~
 
     Test the HTML builder and check output against XPath.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
+import re
+
 import pytest
 
+
 @pytest.mark.sphinx(testroot='toctree-glob')
 def test_relations(app, status, warning):
     app.builder.build_all()
@@ -25,3 +27,21 @@ def test_relations(app, status, warning):
     assert app.builder.relations['qux/qux_1'] == ['qux/index', 'qux/index', 'qux/qux_2']
     assert app.builder.relations['qux/qux_2'] == ['qux/index', 'qux/qux_1', None]
     assert 'quux' not in app.builder.relations
+
+
+@pytest.mark.sphinx('singlehtml', testroot='toctree-empty')
+def test_singlehtml_toctree(app, status, warning):
+    app.builder.build_all()
+    try:
+        app.builder._get_local_toctree('index')
+    except AttributeError:
+        pytest.fail('Unexpected AttributeError in app.builder.fix_refuris')
+
+
+@pytest.mark.sphinx(testroot='toctree', srcdir="numbered-toctree")
+def test_numbered_toctree(app, status, warning):
+    # give argument to :numbered: option
+    index = (app.srcdir / 'index.rst').text()
+    index = re.sub(':numbered:.*', ':numbered: 1', index)
+    (app.srcdir / 'index.rst').write_text(index)
+    app.builder.build_all()
